implementation module System._Windows

import StdEnv
import System.SysCall
import System._Pointer

import code from library "kernel32_library"
import code from library "msvcrt_library"
import code from library "ucrtbase_library"
import code from library "ws2_32_library"

closeHandle :: !HANDLE !*env -> (!Bool,!*env) | SysCallEnv env
closeHandle handle world
	= code {
		ccall CloseHandle@4 "PI:I:I"
	}

setHandleInformation :: !HANDLE !DWORD !DWORD !*env -> (!Bool, !*env) | SysCallEnv env
setHandleInformation hObject dwMask dwFlags world = code {
	ccall SetHandleInformation@12 "PIII:I:I"
}

createDirectoryA :: !String !LPSECURITY_ATTRIBUTES !*env -> (!Bool, !*env) | SysCallEnv env
createDirectoryA lpFileName lpSecurityAttributes world
	= code {
		ccall CreateDirectoryA@8 "PsI:I:I"
	}
	
createFileA :: !String !DWORD !DWORD !LPSECURITY_ATTRIBUTES !DWORD !DWORD !HANDLE !*env -> (!HANDLE, !*env) | SysCallEnv env
createFileA lpFileName dwDesiredAccess dwShareMode lpSecurityAttributes 
	dwCreationDisposition dwFlagsAndAttributes hTemplateFile world
	= code {
		ccall CreateFileA@28 "PsIIIIII:I:I"
	}
	
readFile :: !HANDLE !LPVOID !DWORD !LPDWORD !LPOVERLAPPED !*env -> (!Bool, !*env) | SysCallEnv env
readFile hFile lpBuffer nNumberOfBytesToRead lpNumberOfBytesRead lpOverlapped world
	= code {
		ccall ReadFile@20 "PIpIpp:I:I"
	}
	
writeFile :: !HANDLE !String !DWORD !LPDWORD !LPOVERLAPPED !*env -> (!Bool, !*env) | SysCallEnv env
writeFile hFile lpBuffer nNumberOfBytesToWrite lpNumberOfBytesWritten lpOverlapped world
	= code {
		ccall WriteFile@20 "PIsIpp:I:I"
	}
	
setEndOfFile :: !HANDLE !*env -> (!Bool, !*env) | SysCallEnv env
setEndOfFile hFile world
	= code {
		ccall SetEndOfFile@4 "PI:I:I"
	}

lockFileEx :: !HANDLE !DWORD !DWORD !DWORD !DWORD !LPOVERLAPPED !*env -> (!Bool, !*env) | SysCallEnv env
lockFileEx hFile dwFlags dwReserved nNumberOfBytesToLockLow nNumberOfBytesToLockHigh lpOverlapped world
	= code {
		ccall LockFileEx@24 "PIIIIIp:I:I"
	}
	
unlockFile :: !HANDLE !DWORD !DWORD !DWORD !DWORD !*env -> (!Bool, !*env) | SysCallEnv env
unlockFile hFile dwFileOffsetLow dwFileOffsetHigh nNumberOfBytesToUnlockLow nNumberOfBytesToUnlockHigh world
	= code {
		ccall UnlockFile@20 "PIIIII:I:I"
	}
	
getFileSize :: !HANDLE !LPDWORD !*env -> (!DWORD, !*env) | SysCallEnv env
getFileSize hFile lpFileSizeHigh world
	= code {
		ccall GetFileSize@8 "PIA:I:I"
	}

getFullPathNameA :: !String !DWORD !String !LPTSTR !*env -> (!DWORD, !*env) | SysCallEnv env
getFullPathNameA lpFileName nBufferLnegth lpBuffer lpFilePart world
	= code {
		ccall GetFullPathNameA@16 "PsIsp:I:I"
	}

initializeProcThreadAttributeList :: !LPPROC_THREAD_ATTRIBUTE_LIST !Int !Int !Int !*env -> (!Bool, !*env) | SysCallEnv env
initializeProcThreadAttributeList lpAttributeList dwAttributeCount dwFlags lpSize w = code {
	ccall InitializeProcThreadAttributeList@16 "PpIII:I:A"
}

updateProcThreadAttribute :: !LPPROC_THREAD_ATTRIBUTE_LIST !Int !Int !LPVOID !Int !LPVOID !LPVOID !*env -> (!Bool, !*env) | SysCallEnv env
updateProcThreadAttribute lpAttributeList dwFlags attribute lpValue cbSize lpPreviousValue lpReturnSize w = code {
	ccall UpdateProcThreadAttribute@28 "PpIIpIpp:I:A"
}

deleteProcThreadAttributeList :: !LPPROC_THREAD_ATTRIBUTE_LIST !*env -> *env | SysCallEnv env
deleteProcThreadAttributeList lpAttributeList w = code {
	ccall DeleteProcThreadAttributeList@4 "Pp:V:A"
}

createProcessA :: !String !String !LPSECURITY_ATTRIBUTES !LPSECURITY_ATTRIBUTES !Bool !Int !LPVOID
					!LPCTSTR !LPSTARTUPINFO !LPPROCESS_INFORMATION !*env -> (!Bool,!*env) | SysCallEnv env
createProcessA lpApplicationName commandLine lpProcessAttributes lpThreadAttributes inheritHandles creationFlags lpEnvironment
					currentDirectory lpStartupInfo lpProcessInformation os
	= code {
		ccall CreateProcessA@40 "PssIIIIIIAp:I:I"
	}

createProcessA_dir :: !String !String !LPSECURITY_ATTRIBUTES !LPSECURITY_ATTRIBUTES !Bool !Int !LPVOID
					!String !LPSTARTUPINFO !LPPROCESS_INFORMATION !*env -> (!Bool,!*env) | SysCallEnv env
createProcessA_dir lpApplicationName commandLine lpProcessAttributes lpThreadAttributes inheritHandles creationFlags lpEnvironment
					currentDirectory lpStartupInfo lpProcessInformation os
	= code {
		ccall CreateProcessA@40 "PssIIIIIsAp:I:I"
	}

createProcessA_noApplication :: !Int !String !LPSECURITY_ATTRIBUTES !LPSECURITY_ATTRIBUTES !Bool !Int !LPVOID
					!LPCTSTR !LPSTARTUPINFO !LPPROCESS_INFORMATION !*env -> (!Bool,!*env) | SysCallEnv env
createProcessA_noApplication lpApplicationName commandLine lpProcessAttributes lpThreadAttributes inheritHandles creationFlags lpEnvironment
					currentDirectory lpStartupInfo lpProcessInformation os
	= code {
		ccall CreateProcessA@40 "PIsIIIIIIAp:I:I"
	}

createProcessA_noApplication_dir :: !Int !String !LPSECURITY_ATTRIBUTES !LPSECURITY_ATTRIBUTES !Bool !Int !LPVOID
					!String !LPSTARTUPINFO !LPPROCESS_INFORMATION !*env -> (!Bool,!*env) | SysCallEnv env
createProcessA_noApplication_dir lpApplicationName commandLine lpProcessAttributes lpThreadAttributes inheritHandles creationFlags lpEnvironment
					currentDirectory lpStartupInfo lpProcessInformation os
	= code {
		ccall CreateProcessA@40 "PIsIIIIIsAp:I:I"
	}

terminateProcess :: !HANDLE !Int !*env -> (!Bool, !*env) | SysCallEnv env
terminateProcess hProc exitCode world = code inline {
	ccall TerminateProcess@8 "PII:I:I"
}

deleteFileA :: !String !*env -> (!Int, !*env) | SysCallEnv env
deleteFileA path world = code inline {
	ccall DeleteFileA@4 "Ps:I:I"
}

fileTimeToSystemTime :: !FILETIME !LPSYSTEMTIME !*env -> (!Bool, *env) | SysCallEnv env
fileTimeToSystemTime lpFileTime lpSystemTime world
	= code {
		ccall FileTimeToSystemTime@8 "Pss:I:I"
	}

findClose :: !HANDLE !*env -> (!Bool, !*env) | SysCallEnv env
findClose handle world
	= code {
		ccall FindClose@4 "PI:I:I"
	}

findFirstFileA :: !String !LPWIN32_FIND_DATA !*env -> (!HANDLE, !*env) | SysCallEnv env
findFirstFileA filename win32FindData world
	= code {
		ccall FindFirstFileA@8 "Pss:I:I"
	}

findNextFileA :: !HANDLE !LPWIN32_FIND_DATA !*env -> (!Bool, !*env) | SysCallEnv env
findNextFileA hFindFile lpFindFileData world
	= code {
		ccall FindNextFileA@8 "PIs:I:I"
	}

formatMessageA :: !DWORD !LPCVOID !DWORD !DWORD !{#LPTSTR} !DWORD !Int !*env -> (!DWORD, !*env) | SysCallEnv env
formatMessageA dwFlags lpSource dwMessageId dwLanguageId lpBuffer nSize args w
	= code {
		ccall FormatMessageA@28 "PIIIIAII:I:A"
	}
	
getCurrentDirectoryA :: !DWORD !{#Char} !*env -> (!DWORD, *env) | SysCallEnv env
getCurrentDirectoryA nBufferLength lpBuffer world
	= code {
		ccall GetCurrentDirectoryA@8 "PIs:I:I"
	}

getExitCodeProcess :: !HANDLE !*env -> (!Bool,!Int,!*env) | SysCallEnv env
getExitCodeProcess handle world
	= code {
		ccall GetExitCodeProcess@8 "PI:II:I"
	}

getLastError :: !*env -> (!Int, !*env) | SysCallEnv env
getLastError world
	= code {
		ccall GetLastError@0 "P:I:A"
	}

localFree :: !HLOCAL -> HLOCAL
localFree hMem
	= code {
		ccall LocalFree@4 "PI:I"
	}

moveFileA :: !String !String !*env -> (!Bool, !*env) | SysCallEnv env
moveFileA lpExistingFileName lpNewFileName world
	= code {
		ccall MoveFileA@8 "Pss:I:I"
	}

moveFileExA :: !String !String !Int !*env -> (!Bool, !*env) | SysCallEnv env
moveFileExA lpExistingFileName lpNewFileName dwFlags world
	= code {
		ccall MoveFileExA@12 "PssI:I:I"
	}

copyFileA :: !String !String !Bool !*env -> (!Bool, !*env) | SysCallEnv env
copyFileA lpExistingFileName lpNewFileName bFailIfExists world
	= code {
		ccall CopyFileA@12 "PssI:I:I"
	}
	
removeDirectoryA :: !String !*env -> (!Bool, !*env) | SysCallEnv env
removeDirectoryA lpFileName world
	= code {
		ccall RemoveDirectoryA@4 "Ps:I:I"
	}

setCurrentDirectoryA :: !String !*env -> (!Bool, !*env) | SysCallEnv env
setCurrentDirectoryA lpPathName world
	= code {
		ccall SetCurrentDirectoryA@4 "Ps:I:I"
	}

waitForSingleObject :: !HANDLE !Int !*env -> (!Int,!*env) | SysCallEnv env
waitForSingleObject handle timeout world
	= code {
		ccall WaitForSingleObject@8 "PpI:I:I"
	}

waitForMultipleObjects :: !Int !{#HANDLE} !Bool !Int !*env -> (!Int, !*env) | SysCallEnv env
waitForMultipleObjects nCount lpHandles bWaitAll dwMilliseconds world
	= code {
		ccall WaitForMultipleObjects@16 "PIAII:I:A"
	}

getProcessHeap :: !*env -> (!HANDLE, !*env) | SysCallEnv env
getProcessHeap world
	= code {
		ccall GetProcessHeap@0 "P:I:I"
	}
	
heapAlloc :: !HANDLE !DWORD !SIZE_T !*env -> (!LPVOID, !*env) | SysCallEnv env
heapAlloc hHeap dwFlags dwBytes world
	= code {
		ccall HeapAlloc@12 "PIII:p:I"
	}
	
heapFree :: !HANDLE !DWORD !LPVOID !*env -> (!Bool, !*env) | SysCallEnv env
heapFree hHeap dwFlags lpMem world
	= code {
		ccall HeapFree@12 "PIIp:I:I"
	}

heapCreate :: !DWORD !SIZE_T !SIZE_T !*env -> (!HANDLE, !*env) | SysCallEnv env
heapCreate flOptions dwInitialSize dwMaximumSize world = code {
	ccall HeapCreate@12 "PIII:I:I"
}
	
CreateThread :: !LPSECURITY_ATTRIBUTES !SIZE_T !LPTHREAD_START_ROUTINE !LPVOID !DWORD !*env -> (!HANDLE,!DWORD,!*env) | SysCallEnv env
CreateThread threadAttributes stackSize startAddress parameter creationFlags world = code {
	ccall CreateThread@24 "PpIppI:II:I"
}

ResumeThread :: !HANDLE !*env -> (!DWORD, *env) | SysCallEnv env
ResumeThread threadHandle world = code {
	ccall ResumeThread@4 "PI:I:I"
}

TerminateThread :: !HANDLE !DWORD !*env -> (!Bool, *env) | SysCallEnv env
TerminateThread hThread dwExitCode world = code {
	ccall TerminateThread@8 "PII:I:A"
}

getExitCodeThread :: !HANDLE !*env -> (!Bool,!Int,!*env) | SysCallEnv env
getExitCodeThread handle world = code {
	ccall GetExitCodeThread@8 "Pp:II:A"
}

initializeCriticalSection :: !LPCRITICAL_SECTION !*env -> *env | SysCallEnv env
initializeCriticalSection lpCriticalSection world = code {
	fill_a 0 1
	pop_a 1
	ccall InitializeCriticalSection@4 "PI:V:A"
}

WinGetThreadId :: !HANDLE !*env -> (!DWORD, !*env) | SysCallEnv env
WinGetThreadId handle world = code {
	ccall GetThreadId@4 "Pp:I:I"
}

WinGetCurrentThreadId :: !*env -> (!DWORD, !*env) | SysCallEnv env
WinGetCurrentThreadId world = code {
	ccall GetCurrentThreadId@0 "P:I:I"
}

WinOpenThread :: !DWORD !Bool !DWORD *env -> (!DWORD, !*env) | SysCallEnv env
WinOpenThread dwDesiredAccess bInheritHandle dwThreadId world
 = code {
	ccall OpenThread@12 "PIII:I:I"
}

initializeCriticalSectionAndSpinCount :: !LPCRITICAL_SECTION !DWORD !*env -> (!Bool, !*env) | SysCallEnv env
initializeCriticalSectionAndSpinCount lpCriticalSection dwSpinCount world = code {
	ccall InitializeCriticalSectionAndSpinCount@8 "PpI:I:I"
}

enterCriticalSection :: !LPCRITICAL_SECTION !*env -> *env | SysCallEnv env
enterCriticalSection lpCriticalSection world = code {
	fill_a 0 1
	pop_a 1
	ccall EnterCriticalSection@4 "Pp:V:I"
}
	
leaveCriticalSection :: !LPCRITICAL_SECTION !*env -> *env | SysCallEnv env
leaveCriticalSection lpCriticalSection world = code {
	fill_a 0 1
	pop_a 1
	ccall LeaveCriticalSection@4 "Pp:V:I"
}

createMutexA :: !LPSECURITY_ATTRIBUTES !Bool !LPCTSTR !*env -> (!HANDLE, !*env) | SysCallEnv env
createMutexA lpMutexAttributes bInitialOwner lpName world = code {
	ccall CreateMutexA@12 "PpIp:I:I"
}

releaseMutex :: !HANDLE !*env -> (!Bool, !*env) | SysCallEnv env
releaseMutex hMutex world = code {
	ccall ReleaseMutex@4 "PI:I:I"
}

createEventA :: !LPSECURITY_ATTRIBUTES !Bool !Bool !LPCTSTR !*env -> (!HANDLE, !*env) | SysCallEnv env
createEventA lpEventAttributes bManualReset bInitialState lpName world = code {
	ccall CreateEventA@16 "PpIIp:I:I"
}

setEvent :: !HANDLE !*env -> (!Bool, !*env) | SysCallEnv env
setEvent hEvent world = code {
	ccall SetEvent@4 "PI:I:I"
}

sleep :: !DWORD !*env -> *env | SysCallEnv env
sleep dwMilliseconds world = code {
	ccall Sleep@4 "PI:V:A"
}

createPipe :: !PHANDLE !PHANDLE !SECURITY_ATTRIBUTES !DWORD !*env -> (!Bool, !*env) | SysCallEnv env
createPipe hReadPipe hWritePipe lpPipeAttributes nSize world = code {
	ccall CreatePipe@16 "PppAI:I:I"
}

peekNamedPipe :: !HANDLE !LPVOID !DWORD !LPDWORD !LPDWORD !LPDWORD !*env -> (!Bool, !*env) | SysCallEnv env
peekNamedPipe hNamedPipe lpBuffer nBufferSize lpBytesRead lpTotalBytesAvail lpBytesLeftThisMessage world = code {
	ccall PeekNamedPipe@24 "PIpIppp:I:I"
}

cancelSynchronousIo :: !HANDLE !*env -> (!Bool, !*env) | SysCallEnv env
cancelSynchronousIo hThread w = code {
	ccall CancelSynchronousIo@4 "Pp:I:A"
}

GetSystemTimeAsFileTime :: !{#Int} !*env -> (!{#Int},!*env) | SysCallEnv env
GetSystemTimeAsFileTime i w
= code {
	push_a 0
	ccall GetSystemTimeAsFileTime@4 "PA:I:AA"
	pop_b 1
}

GetEnvironmentVariableA :: !{#Char} !{#Char} !Int !*env -> (!Int, !*env) | SysCallEnv env
GetEnvironmentVariableA a0 a1 a2 w = code {
	ccall GetEnvironmentVariableA@12 "PssI:I:A"
}

SetEnvironmentVariableA :: !{#Char} !Int !*env -> (!Int, !*env) | SysCallEnv env
SetEnvironmentVariableA a0 a1 a2 = code {
	ccall SetEnvironmentVariableA@8 "PsI:I:A"
}

_mkgmtime :: !{#Int} !*env -> (!Int, !*env) | SysCallEnv env
_mkgmtime tm w = code {
	ccall _mkgmtime "A:I:A"
}

WSAStartup :: !Int !{#Int} !*env -> (!Int, !*env) | SysCallEnv env
WSAStartup a0 a1 a2 = code {
	ccall WSAStartup@8 "PIA:I:A"
}
WSACleanup :: !*env -> (!Int, !*env) | SysCallEnv env
WSACleanup a0 = code {
	ccall WSACleanup@0 "P:I:A"
}
gethostbyname :: !{#Char} !*env -> (!Pointer, !*env) | SysCallEnv env
gethostbyname a0 a1 = code {
	ccall gethostbyname@4 "Ps:I:A"
}

malloc :: !Int !*env -> (!Pointer, !*env) | SysCallEnv env
malloc num w = code {
	ccall malloc "I:p:A"
}

free :: !Pointer !*env -> *env | SysCallEnv env
free ptr world = code {
   ccall free "p:V:A"
}

memcpy :: !Pointer !Pointer !Int !*env -> *env | SysCallEnv env
memcpy s p n w = code {
    ccall memcpy "ppI:V:A"
}

memcpyStringToPointer :: !Pointer !{#Char} !Int !*env -> *env | SysCallEnv env
memcpyStringToPointer p s n w = code {
    ccall memcpy "psI:V:A"
}

memcpyPointerToString :: !{#Char} !Pointer !Int !*env -> *env | SysCallEnv env
memcpyPointerToString s p n w = code {
    ccall memcpy "spI:V:A"
}

memcmp :: !Pointer !Pointer !Int !*env -> (!Int, !*env) | SysCallEnv env
memcmp ptr1 ptr2 num w = code {
	ccall memcmp "ppI:I:A"
}
_execvp :: !{#Char} !{#Pointer} !*env -> (!Int, !*env) | SysCallEnv env
_execvp path args w = code { ccall _execvp "sA:I:A"}
